@@ -407,13 +407,16 @@ class MRPServer:
 
     async def handle_execute_stream(self, request: Request) -> EventSourceResponse:
         """POST /execute/stream - SSE streaming execution"""
+        print(f"[DEBUG] handle_execute_stream called", flush=True)
         body = await request.json()
         code = body.get("code", "")
         session_id = body.get("session", "default")
         store_history = body.get("storeHistory", True)
         exec_id = body.get("execId", str(uuid.uuid4())[:8])
+        print(f"[DEBUG] session_id={session_id}, code length={len(code)}, exec_id={exec_id}", flush=True)
 
         worker, session = self.session_manager.get_or_create_session(session_id)
+        print(f"[DEBUG] Got worker, starting event generator", flush=True)
 
         async def event_generator():
             # Capture the event loop for use in background threads
@@ -568,9 +571,29 @@ class MRPServer:
         return JSONResponse({"cancelled": False, "error": "No pending input request"})
 
     async def handle_interrupt(self, request: Request) -> JSONResponse:
-        """POST /interrupt"""
-        # TODO: Implement actual interrupt via signal
-        return JSONResponse({"interrupted": True})
+        """POST /interrupt
+
+        Interrupt currently running code in a session.
+        Sends SIGINT to subprocess workers or sets interrupt flag for local workers.
+        """
+        body = await request.json()
+        session_id = body.get("session", "default")
+
+        # Get the worker for this session (don't create if doesn't exist)
+        session = self.session_manager.get_session(session_id)
+        if not session:
+            return JSONResponse({"interrupted": False, "error": "Session not found"})
+
+        worker = self.session_manager.workers.get(session_id)
+        if not worker:
+            return JSONResponse({"interrupted": False, "error": "Worker not found"})
+
+        # Call interrupt on the worker
+        try:
+            interrupted = worker.interrupt()
+            return JSONResponse({"interrupted": interrupted})
+        except Exception as e:
+            return JSONResponse({"interrupted": False, "error": str(e)})
 
     async def handle_complete(self, request: Request) -> JSONResponse:
         """POST /complete"""
