@@ -78,6 +78,10 @@ class IPythonWorker:
         self._captured_displays: list[dict] = []
         self._asset_counter = 0
         self._interrupt_requested = False
+
+        # Set matplotlib backend to Agg early (before any imports)
+        # This ensures plots work even if matplotlib is installed later via %pip
+        os.environ.setdefault("MPLBACKEND", "Agg")
         self._current_exec_id: str | None = None
         self._pending_input: dict | None = None  # For stdin_request handling
         self._input_event: threading.Event | None = None
@@ -742,7 +746,13 @@ class IPythonWorker:
         result = ExecuteResult()
 
         try:
+            # Ensure matplotlib hook is applied BEFORE execution
+            # (handles case where matplotlib was installed via %pip)
+            self._ensure_matplotlib_hook()
+
             exec_result = self.shell.run_cell(code, store_history=store_history, silent=False)
+
+            # Also check after, in case user imported matplotlib during this cell
             self._ensure_matplotlib_hook()
 
             result.executionCount = self.shell.execution_count
@@ -988,7 +998,13 @@ class IPythonWorker:
         getpass_module.getpass = hooked_getpass
 
         try:
+            # Ensure matplotlib hook is applied BEFORE execution
+            # (handles case where matplotlib was installed via %pip)
+            self._ensure_matplotlib_hook()
+
             exec_result = self.shell.run_cell(code, store_history=store_history, silent=False)
+
+            # Also check after, in case user imported matplotlib during this cell
             self._ensure_matplotlib_hook()
 
             result.executionCount = self.shell.execution_count
@@ -1141,10 +1157,22 @@ class IPythonWorker:
             if not name:
                 return InspectResult(found=False)
 
-            info = self.shell.object_inspect(name)
+            # detail_level=1 tells IPython to include source code
+            ipython_detail = 1 if detail >= 2 else 0
+            info = self.shell.object_inspect(name, detail_level=ipython_detail)
             if not info.get("found"):
                 return InspectResult(found=False, name=name)
 
+            # If IPython didn't get source, try inspect.getsource() directly
+            source_code = info.get("source")
+            if detail >= 2 and not source_code:
+                try:
+                    import inspect
+                    obj = eval(name, {"__builtins__": {}}, self.shell.user_ns)
+                    source_code = inspect.getsource(obj)
+                except Exception:
+                    pass
+
             return InspectResult(
                 found=True,
                 source="runtime",
@@ -1153,7 +1181,7 @@ class IPythonWorker:
                 type=info.get("type_name"),
                 signature=info.get("call_signature") or info.get("init_signature"),
                 docstring=info.get("docstring") if detail >= 1 else None,
-                sourceCode=info.get("source") if detail >= 2 else None,
+                sourceCode=source_code if detail >= 2 else None,
                 file=info.get("file"),
                 line=info.get("line"),
             )
